#include "SamplePalette.hpp"


SamplePalette* SamplePalette::instance = nullptr;


SamplePalette::SamplePalette() 
	: DG::Palette(ACAPI_GetOwnResModule(), 32500, ACAPI_GetOwnResModule(), SamplePaletteGuid)
	, listBox(GetReference(), listBoxID)
{
	Attach(*this);
	BeginEventProcessing();
	listBox.Attach(*this);
}

SamplePalette::~SamplePalette()
{
	Detach(*this);
	EndEventProcessing();
}

SamplePalette& SamplePalette::GetInstance()
{
	return *instance;
}

bool SamplePalette::HasInstance()
{
	return instance != nullptr;
}

void SamplePalette::CreateInstance()
{
	instance = new SamplePalette;
	ACAPI_KeepInMemory(true);
}

void SamplePalette::ShowPalette()
{
	if (!IsPaletteVisible())
	{
		DG::Palette::Show();
		DG::Palette::BringToFront();

		GetElementList();
	}
}

void SamplePalette::HidePalette()
{
	instance->Hide();
}

void SamplePalette::ClosePalette()
{
	if (instance != nullptr)
	{
		HidePalette();
		DeleteUserData();

		delete instance;
		instance = nullptr;
	}
}

bool SamplePalette::IsPaletteVisible()
{
	return instance != nullptr && instance->IsVisible();
}

void SamplePalette::DeleteUserData()
{
	for (int i = 1; i <= listBox.GetItemCount(); i++)
	{
		API_Guid* pGuid = (API_Guid*)listBox.GetItemValue(i);
		if (pGuid != nullptr) {
			delete pGuid;
		}
	}
}


void SamplePalette::GetElementList()
{
	if (listBox.GetItemCount() > 0) {
		DeleteUserData();
		listBox.DeleteItem(DG::ListBox::AllItems);
	}

	GSErrCode err = NoError;
	GS::Array<API_Guid> elemGuids;

	err = ACAPI_Element_GetElemList(API_ZombieElemID, &elemGuids);
	if (err != NoError)
		return;

	for (API_Guid guid : elemGuids)
	{
		API_Element elem = {};
		elem.header.guid = guid;

		err = ACAPI_Element_Get(&elem);
		if (err != NoError)
			continue;

		listBox.AppendItem();
		GS::UniString text = ElemID_To_Name(elem.header.typeID);
		listBox.SetTabItemText(DG::SingleSelListBox::BottomItem, 1, text);

		// GuidXg{bNXACeɕt
		API_Guid* pGuid = new API_Guid(guid);
		listBox.SetItemValue(DG::SingleSelListBox::BottomItem, (DGUserData)pGuid);
	}
}


void SamplePalette::PanelOpened(const DG::PanelOpenEvent& ev)
{
	UNUSED_VARIABLE(ev);
}

void SamplePalette::PanelCloseRequested(const DG::PanelCloseRequestEvent& ev, bool* accepted)
{
	UNUSED_VARIABLE(ev);
	HidePalette();
	*accepted = true;
}

void SamplePalette::ListBoxClicked(const DG::ListBoxClickEvent& ev)
{
	if (ev.GetSource() == &listBox)
	{
		short selecedItem = listBox.GetSelectedItem();

		API_Guid* pGuid = (API_Guid*)listBox.GetItemValue(selecedItem);
		if (pGuid == nullptr)
			return;

		API_Neig neig = {};
		neig.guid = *pGuid;
		ACAPI_Element_DeselectAll();
		ACAPI_Element_Select({ neig }, true);

		GS::Array<API_Guid> guidArray;
		guidArray.Push(*pGuid);
		ACAPI_Automate(APIDo_ZoomToElementsID, &guidArray);
	}
}
